/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Device.hpp"
#include "Omni.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <dlfcn.h>

int
handleDevice (char *pszFullDeviceName)
{
   Device       *pDevice      = 0;
   PFNNEWDEVICE  pfnNewDevice = 0;
   void         *vhDevice     = 0;

   cout << "Trying to load " << pszFullDeviceName << endl;

   vhDevice = dlopen (pszFullDeviceName, RTLD_NOW | RTLD_GLOBAL);

   if (DebugOutput::shouldOutputDeviceTester2 ()) cerr << "vhDevice = " << hex << vhDevice << endl;

   if (!vhDevice)
   {
      cerr << "dlerror returns " << dlerror () << endl;
      return 2;
   }

   pfnNewDevice = (PFNNEWDEVICE)dlsym (vhDevice, "newDevice__Fb");

   if (DebugOutput::shouldOutputDeviceTester2 ()) cerr << hex << "pfnNewDevice = 0x" << (int)pfnNewDevice << endl;

   if (!pfnNewDevice)
      return 3;

   pDevice = pfnNewDevice (true);

   if (DebugOutput::shouldOutputDeviceTester2 ()) cerr << "pDevice = " << *pDevice << endl;

   DeviceOrientation *pDO   = pDevice->getCurrentOrientation ();
   Enumeration       *pEnum = pDO->getEnumeration ();

   cout << "Enumerating orientations:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDO = (DeviceOrientation *)pEnum->nextElement ();

      cout << DeviceOrientation::IDToName (pDO->getID ()) << endl;

      delete pDO;
   }
   cout << "Done!" << endl;

   delete pEnum;

   DeviceForm  *pDF = pDevice->getCurrentForm ();
   pEnum = pDF->getEnumeration ();

   cout << "Enumerating forms:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDF = (DeviceForm *)pEnum->nextElement ();

      cout << DeviceForm::IDToName (pDF->getID ()) << endl;

      delete pDF;
   }
   cout << "Done!" << endl;

   delete pEnum;

   DeviceTray *pDT = pDevice->getCurrentTray ();
   pEnum = pDT->getEnumeration ();

   cout << "Enumerating trays:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDT = (DeviceTray *)pEnum->nextElement ();

      cout << DeviceTray::IDToName (pDT->getID ()) << endl;

      delete pDT;
   }
   cout << "Done!" << endl;

   delete pEnum;

   DeviceMedia *pDM = pDevice->getCurrentMedia ();
   pEnum = pDM->getEnumeration ();

   cout << "Enumerating medias:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDM = (DeviceMedia *)pEnum->nextElement ();

      cout << DeviceMedia::IDToName (pDM->getID ()) << endl;

      delete pDM;
   }
   cout << "Done!" << endl;

   delete pEnum;

   DeviceResolution *pDR = pDevice->getCurrentResolution ();
   pEnum = pDR->getEnumeration ();

   cout << "Enumerating resolutions:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDR = (DeviceResolution *)pEnum->nextElement ();

      cout << DeviceResolution::IDToName (pDR->getID ()) << endl;

      delete pDR;
   }
   cout << "Done!" << endl;

   delete pEnum;

   DevicePrintMode *pDPM = pDevice->getCurrentPrintMode ();
   pEnum = pDPM->getEnumeration ();

   cout << "Enumerating print modes:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDPM = (DevicePrintMode *)pEnum->nextElement ();

      cout << DevicePrintMode::IDToName (pDPM->getID ()) << endl;

      delete pDPM;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = DeviceDither::getEnumeration ();

   cout << "Enumerating dithering ids:" << endl;
   while (pEnum->hasMoreElements ())
   {
      char *pszID = (char *)pEnum->nextElement ();

      cout << pszID << endl;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDevice->listDeviceJobPropertyKeys ();

   cout << "Enumerating device job properties:" << endl;
   while (pEnum->hasMoreElements ())
   {
      char *pszKey = (char *)pEnum->nextElement ();

      if (!pszKey)
      {
         cerr << "Error: Enumeration returns null!" << endl;
         break;
      }

      string *pstringValue = pDevice->getJobProperty (pszKey);

      if (!pstringValue)
      {
         cerr << "Error: getJobProperty (" << pszKey << ") return null!" << endl;
         break;
      }

      cout << pszKey << " = " << *pstringValue << endl;

      delete pstringValue;
   }
   cout << "Done!" << endl;

   delete pEnum;

   if (pDevice->hasCapability (Capability::COLOR))
   {
      cout << "This device can print in color." << endl;
   }

   if (pDevice->hasCapability (Capability::MONOCHROME))
   {
      cout << "This device can print in monochrome." << endl;
   }

   if (pDevice->hasCapability (Capability::MIRROR))
   {
      cout << "This device can print in mirror image." << endl;
   }

   delete pDevice;

   int rc = 0;

   rc = dlclose (vhDevice);

   if (DebugOutput::shouldOutputDeviceTester2 ()) cerr << dec << "dlclose returns " << rc << endl;

   return 0;
}

int
main (int argc, char *argv[])
{
   int iFailed = 0;

   if (2 != argc)
   {
      cerr << "Usage: " << argv[0] << " [ printer_library_name | all ]" << endl;
      return 1;
   }

   if (0 == strcasecmp (argv[1], "all"))
   {
      Enumeration *pEnum     = Omni::listDevices ();

      while (pEnum->hasMoreElements ())
      {
         char *pszLibName = (char *)pEnum->nextElement ();

         cout << "Checking: " << pszLibName << endl;

         iFailed |= handleDevice (pszLibName);
      }

      delete pEnum;
   }
   else
   {
      char achLibDeviceName[512];  // @TBD

      if (0 != strncasecmp (argv[1], "lib", 3))
      {
         sprintf (achLibDeviceName, "lib%s.so", argv[1]);
      }
      else
      {
         strcpy (achLibDeviceName, argv[1]);
      }

      iFailed |= handleDevice (achLibDeviceName);
   }

   return iFailed;
}
